@@ -1,9 +1,9 @@
 /*****************************************************************************
- * RRDtool 1.3.1  Copyright by Tobi Oetiker, 1997-2008
+ * RRDtool 1.3.3  Copyright by Tobi Oetiker, 1997-2008
  *****************************************************************************
  * rrdlib.h   Public header file for librrd
  *****************************************************************************
- * $Id: rrd.h 1447 2008-07-23 13:02:26Z oetiker $
+ * $Id: rrd.h 1509 2008-09-14 20:32:10Z oetiker $
  * $Log$
  * Revision 1.9  2005/02/13 16:13:33  oetiker
  * let rrd_graph return the actual value range it picked ...
@@ -322,36 +322,53 @@ extern    "C" {
 #if defined(__GNUC__) && defined (RRD_EXPORT_DEPRECATED)
 # define RRD_DEPRECATED __attribute__((deprecated))
 #else
-# define RRD_DEPRECATED /**/
+# define RRD_DEPRECATED          /**/
 #endif
+     void     rrd_free(
+    rrd_t *rrd)
+              RRD_DEPRECATED;
+    void      rrd_init(
+    rrd_t *rrd)
+              RRD_DEPRECATED;
 
-    void        rrd_free     (rrd_t *rrd)
-      RRD_DEPRECATED;
-    void        rrd_init     (rrd_t *rrd)
-      RRD_DEPRECATED;
+    rrd_file_t *rrd_open(
+    const char *const file_name,
+    rrd_t *rrd,
+    unsigned rdwr)
+              RRD_DEPRECATED;
 
-    rrd_file_t *rrd_open   (const char *const file_name, rrd_t *rrd,
-        unsigned rdwr)
-      RRD_DEPRECATED;
-
-    void        rrd_dontneed (rrd_file_t *rrd_file, rrd_t *rrd)
-      RRD_DEPRECATED;
-    int         rrd_close    (rrd_file_t *rrd_file)
-      RRD_DEPRECATED;
-    ssize_t     rrd_read     (rrd_file_t *rrd_file, void *buf, size_t count)
-      RRD_DEPRECATED;
-    ssize_t     rrd_write    (rrd_file_t *rrd_file,
-        const void *buf, size_t count)
-      RRD_DEPRECATED;
-    void        rrd_flush    (rrd_file_t *rrd_file)
-      RRD_DEPRECATED;
-    off_t       rrd_seek     (rrd_file_t *rrd_file, off_t off, int whence)
-      RRD_DEPRECATED;
-    off_t       rrd_tell     (rrd_file_t *rrd_file)
-      RRD_DEPRECATED;
-    int         rrd_lock     (rrd_file_t *file)
-      RRD_DEPRECATED;
-#endif /* defined(_RRD_TOOL_H) || defined(RRD_EXPORT_DEPRECATED) */
+    void      rrd_dontneed(
+    rrd_file_t *rrd_file,
+    rrd_t *rrd)
+              RRD_DEPRECATED;
+    int       rrd_close(
+    rrd_file_t *rrd_file)
+              RRD_DEPRECATED;
+    ssize_t   rrd_read(
+    rrd_file_t *rrd_file,
+    void *buf,
+    size_t count)
+              RRD_DEPRECATED;
+    ssize_t   rrd_write(
+    rrd_file_t *rrd_file,
+    const void *buf,
+    size_t count)
+              RRD_DEPRECATED;
+    void      rrd_flush(
+    rrd_file_t *rrd_file)
+              RRD_DEPRECATED;
+    off_t     rrd_seek(
+    rrd_file_t *rrd_file,
+    off_t off,
+    int whence)
+              RRD_DEPRECATED;
+    off_t     rrd_tell(
+    rrd_file_t *rrd_file)
+              RRD_DEPRECATED;
+    int       rrd_lock(
+    rrd_file_t *file)
+              RRD_DEPRECATED;
+#endif                  /* defined(_RRD_TOOL_H) || defined(RRD_EXPORT_DEPRECATED) */
 
 #endif                  /* _RRDLIB_H */
 
